package nucleus

import (
	tpb "code.fbi.h-da.de/cocsn/api/go/gosdn/transport"
	"code.fbi.h-da.de/cocsn/gosdn/interfaces/device"
	"code.fbi.h-da.de/cocsn/gosdn/interfaces/southbound"
	"code.fbi.h-da.de/cocsn/gosdn/interfaces/transport"
	"github.com/docker/docker/pkg/namesgenerator"
	"github.com/google/uuid"
	"github.com/openconfig/ygot/ygot"
	"google.golang.org/protobuf/proto"
)

// NewDevice creates a Device
func NewDevice(name string, opt *tpb.TransportOption, sbi southbound.SouthboundInterface) (device.Device, error) {
	t, err := NewTransport(opt, sbi)
	if err != nil {
		return nil, err
	}

	if name == "" {
		name = namesgenerator.GetRandomName(0)
	}

	return &CommonDevice{
		UUID:      uuid.New(),
		GoStruct:  sbi.Schema().Root,
		sbi:       sbi,
		transport: t,
		name:      name,
	}, nil
}

// CommonDevice represents an OND
type CommonDevice struct {
	// UUID represents the Devices UUID
	UUID uuid.UUID

	// Device embeds a ygot.GoStruct containing the device details
	ygot.GoStruct

	// SBI is the device's southbound interface implementation
	sbi southbound.SouthboundInterface

	// Transport is the device's Transport implementation
	transport transport.Transport

	// Name is the device's human readable name
	name string
}

// ID returns the UUID of the Device
func (d *CommonDevice) ID() uuid.UUID {
	return d.UUID
}

// Model returns the ygot representation of the Device
func (d *CommonDevice) Model() ygot.GoStruct {
	return d.GoStruct
}

// Transport returns the Transport of the device
func (d *CommonDevice) Transport() transport.Transport {
	return d.transport
}

// Name returns the name of the device
func (d *CommonDevice) Name() string {
	return d.name
}

// SBI returns the sbi of the Device
func (d *CommonDevice) SBI() southbound.SouthboundInterface {
	return d.sbi
}

// SetTransport sets the Device's Transport
func (d *CommonDevice) SetTransport(t transport.Transport) {
	d.transport = t
}

// SetName sets the Device's name
func (d *CommonDevice) SetName(n string) {
	d.name = n
}

// SetSBI sets the Device's SBI
func (d *CommonDevice) SetSBI(sbi southbound.SouthboundInterface) {
	d.sbi = sbi
}

// ProcessResponse processes a response for the Device
func (d *CommonDevice) ProcessResponse(resp proto.Message) error {
	return d.transport.ProcessResponse(resp, d.GoStruct, d.sbi.Schema())
}

type csbiDevice struct {
	CommonDevice
}

// ID returns the UUID of the Device
func (d *csbiDevice) ID() uuid.UUID {
	return d.UUID
}

func (d *csbiDevice) Model() ygot.GoStruct {
	return d.GoStruct
}

func (d *csbiDevice) Transport() transport.Transport {
	return d.transport
}

func (d *csbiDevice) Name() string {
	return d.name
}

func (d *csbiDevice) SBI() southbound.SouthboundInterface {
	return d.sbi
}

func (d *csbiDevice) ProcessResponse(resp proto.Message) error {
	return d.transport.ProcessResponse(resp, d.GoStruct, d.sbi.Schema())
}
