package nucleus

import (
	"testing"

	tpb "code.fbi.h-da.de/cocsn/api/go/gosdn/transport"
	"code.fbi.h-da.de/cocsn/gosdn/interfaces/southbound"
	gpb "github.com/openconfig/gnmi/proto/gnmi"
)

// TestNewTransport is for input validation only. Functional tests
// are conducted at the transport implementation constructors.
func TestNewTransport(t *testing.T) {
	type args struct {
		opts *tpb.TransportOption
		sbi  southbound.SouthboundInterface
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{
			name: "default",
			args: args{
				opts: &tpb.TransportOption{
					Address:  "test",
					Username: "test",
					Password: "test",
					Tls:      false,
					Csbi:     false,
					TransportOption: &tpb.TransportOption_GnmiTransportOption{
						GnmiTransportOption: &tpb.GnmiTransportOption{
							Encoding: gpb.Encoding_PROTO,
						},
					},
				},
				sbi: &OpenConfig{},
			},
			wantErr: false,
		},
		{
			name: "no opt",
			args: args{
				opts: nil,
				sbi:  &OpenConfig{},
			},
			wantErr: true,
		},
		{
			name: "no sbi",
			args: args{
				opts: &tpb.TransportOption{
					Address:  "test",
					Username: "test",
					Password: "test",
					Tls:      false,
					Csbi:     false,
					TransportOption: &tpb.TransportOption_GnmiTransportOption{
						GnmiTransportOption: &tpb.GnmiTransportOption{
							Encoding: gpb.Encoding_PROTO,
						},
					},
				},
				sbi: nil,
			},
			wantErr: true,
		},
		{
			name: "no implementation options",
			args: args{
				opts: &tpb.TransportOption{
					Address:  "test",
					Username: "test",
					Password: "test",
					Tls:      false,
					Csbi:     false,
				},
				sbi: &OpenConfig{},
			},
			wantErr: true,
		},
		{
			name: "no inner implementation options",
			args: args{
				opts: &tpb.TransportOption{
					Address:         "test",
					Username:        "test",
					Password:        "test",
					Tls:             false,
					Csbi:            false,
					TransportOption: &tpb.TransportOption_GnmiTransportOption{},
				},
				sbi: &OpenConfig{},
			},
			wantErr: true,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			_, err := NewTransport(tt.args.opts, tt.args.sbi)
			if (err != nil) != tt.wantErr {
				t.Errorf("NewTransport() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
		})
	}
}
